package com.tanpu.community.manager;

import com.tanpu.common.exception.BizException;
import com.tanpu.community.api.beans.TopicBriefInfoDTO;
import com.tanpu.community.api.beans.TopicDTO;
import com.tanpu.community.api.constants.RedisKeyConstant;
import com.tanpu.community.api.constants.TopicStatusEnum;
import com.tanpu.community.controller.convert.TopicConverter;
import com.tanpu.community.dao.entity.community.ThemeEntity;
import com.tanpu.community.dao.entity.community.TopicEntity;
import com.tanpu.community.service.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.stream.Collectors;

@Service
public class TopicManager {

    @Autowired
    private TopicService topicService;

    @Autowired
    private RedisService redisService;

    @Autowired
    private ThemeService themeService;

    @Autowired
    private CollectionService collectionService;

    @Autowired
    private CommentService commentService;

    public void insertTopic(String topicTitle,String userId){
        TopicEntity topicEntity = new TopicEntity();
        topicEntity.setCreateBy(userId);
        topicEntity.setUpdateBy(userId);
        topicEntity.setTopicTitle(topicTitle);
        topicEntity.setIsTop(TopicStatusEnum.FALSE.getCode());
        topicEntity.setIsConceal(TopicStatusEnum.FALSE.getCode());
        topicService.addTopic(topicEntity);
    }


    public List<TopicEntity> getAllTopic() {
        return topicService.queryTopicList();
    }

    public void setTopTopic(String topicId, boolean setTop) {
        TopicEntity topicEntity = topicService.queryById(topicId);
        if (topicEntity == null){
            throw new BizException("话题id不正确:" + topicId);
        }
        if (setTop){
            topicService.updateTopicToTop(topicId);
        }else {
            topicService.updateTopicNotTop(topicId);
        }
    }

    public void setTopicConceal(String topicId, boolean setConceal) {
        TopicEntity topicEntity= topicService.queryById(topicId);
        if (topicEntity==null){
            throw new BizException("话题id不正确:"+topicId);
        }
        if (setConceal){
            topicService.updateTopicToConceal(topicId);
        }else {
            topicService.updateTopicNotConceal(topicId);
        }
    }

    public TopicDTO getDetail(String topicId) {
        TopicEntity topicEntity = topicService.queryById(topicId);
        TopicDTO topicDTO = TopicConverter.convertToDTO(topicEntity);
        //TODO：添加实时数据(浏览量，点赞量，评论量，收藏量)
        topicDTO.setViewAmount(redisService.getLong(RedisKeyConstant.TOPIC_VIEW_AMOUNT_+topicId));
        topicDTO.setLikeAmount(redisService.getLong(RedisKeyConstant.TOPIC_LIKE_AMOUNT_+topicId));
        topicDTO.setUserAmount(redisService.getLong(RedisKeyConstant.TOPIC_USER_AMOUNT_+topicId));
        return topicDTO;
    }

    public void modifyPV(String topicId, Long modifyMount) {
        if (modifyMount>0){
            redisService.incr(RedisKeyConstant.TOPIC_VIEW_AMOUNT_+topicId,modifyMount);
        }else {
            redisService.decr(RedisKeyConstant.TOPIC_VIEW_AMOUNT_+topicId,-modifyMount);
        }

    }



    public void calculate(){
        List<String> topicIds = topicService.queryTopicList().stream().map(TopicEntity::getId).collect(Collectors.toList());
        for (String topicId : topicIds) {
            List<ThemeEntity> themeEntities = themeService.selectByTopic(topicId);
            List<String> themeIds = themeEntities.stream().map(ThemeEntity::getId).collect(Collectors.toList());
            Long likeAmountByThemeIds = collectionService.getLikeAmountByThemeIds(themeIds);
            Long bookAmountByThemeIds = collectionService.getBookAmountByThemeIds(themeIds);
            Long commentAmountByThemeIds = commentService.getCommentAmountByThemeIds(themeIds);
            redisService.set(RedisKeyConstant.TOPIC_LIKE_AMOUNT_+topicId, likeAmountByThemeIds);
            redisService.set(RedisKeyConstant.TOPIC_BOOK_AMOUNT_+topicId, bookAmountByThemeIds);
            redisService.set(RedisKeyConstant.TOPIC_COMMENT_AMOUNT_+topicId, commentAmountByThemeIds);
        }

    }

    public List<TopicBriefInfoDTO> getBreifInfoTopic() {
        topicService.queryTopicList();
        return null;

    }
}
